Package: monocle
Type: Package
Title: Clustering, differential expression, and trajectory analysis for
        single- cell RNA-Seq
Version: 2.41.0
Date: 2024-03-13
Author: Cole Trapnell
Maintainer: Cole Trapnell <coletrap@uw.edu>
Description: Monocle performs differential expression and time-series analysis
    for single-cell expression experiments. It orders individual cells
    according to progress through a biological process, without knowing ahead
    of time which genes define progress through that process. Monocle also
    performs differential expression analysis, clustering, visualization, and
    other useful tasks on single cell expression data.  It is designed to work
    with RNA-Seq and qPCR data, but could be used with other types as well.
License: Artistic-2.0
Depends: R (>= 2.10.0), methods, Matrix (>= 1.2-6), Biobase, ggplot2
        (>= 1.0.0), VGAM (>= 1.0-6), DDRTree (>= 0.1.4),
Imports: parallel, igraph (>= 1.0.1), BiocGenerics, HSMMSingleCell (>=
        0.101.5), plyr, cluster, combinat, fastICA, grid, irlba (>=
        2.0.0), matrixStats, Rtsne, MASS, reshape2, leidenbase (>=
        0.1.9), limma, tibble, dplyr, pheatmap, stringr, proxy, slam,
        viridis, stats, biocViews, RANN(>= 2.5), Rcpp (>= 0.12.0)
LinkingTo: Rcpp
Suggests: destiny, Hmisc, knitr, Seurat, scater, testthat
VignetteBuilder: knitr
Roxygen: list(wrap = FALSE)
LazyData: true
biocViews: ImmunoOncology, Sequencing, RNASeq, GeneExpression,
        DifferentialExpression, Infrastructure, DataImport,
        DataRepresentation, Visualization, Clustering,
        MultipleComparison, QualityControl
Packaged: 2026-04-28 23:52:18 UTC; biocbuild
RoxygenNote: 7.3.1
git_url: https://git.bioconductor.org/packages/monocle
git_branch: devel
git_last_commit: 733301c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
