% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{load_HSMM}
\alias{load_HSMM}
\title{Build a CellDataSet from the HSMMSingleCell package}
\usage{
load_HSMM()
}
\description{
Creates a cellDataSet using the data from the
HSMMSingleCell package.
}
