% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-CellDataSet.R
\docType{methods}
\name{reducedDimK}
\alias{reducedDimK}
\title{Retrieves the the whitening matrix during independent component analysis.}
\usage{
reducedDimK(cds)
}
\arguments{
\item{cds}{A CellDataSet object.}
}
\value{
A matrix, where each row is a set of whitened expression values for a feature and columns are cells.
}
\description{
Retrieves the the whitening matrix during independent component analysis.
}
\examples{
\dontrun{
K <- reducedDimW(HSMM)
}
}
