% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-CellDataSet.R
\name{reducedDimA<-}
\alias{reducedDimA<-}
\title{Get the weights needed to lift cells back to high dimensional expression space.}
\usage{
reducedDimA(cds) <- value
}
\arguments{
\item{cds}{A CellDataSet object.}

\item{value}{A whitened expression data matrix}
}
\value{
An updated CellDataSet object
}
\description{
Sets the weights transform the cells' coordinates in the reduced dimension 
space back to the full (whitened) space.
}
\examples{
\dontrun{
cds <- reducedDimA(A)
}
}
