% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expr_models.R
\name{fit_model_helper}
\alias{fit_model_helper}
\title{Helper function for parallel VGAM fitting}
\usage{
fit_model_helper(
  x,
  modelFormulaStr,
  expressionFamily,
  relative_expr,
  disp_func = NULL,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{x}{test}

\item{modelFormulaStr}{a formula string specifying the model to fit for the genes.}

\item{expressionFamily}{specifies the VGAM family function used for expression responses}

\item{relative_expr}{Whether to transform expression into relative values}

\item{disp_func}{test}

\item{verbose}{Whether to show VGAM errors and warnings. Only valid for cores = 1.}

\item{...}{test}
}
\description{
test
}
